#include "ScanAreaHeight.hpp"
#include "Scanner.hpp"
#include "Resolution.hpp"
#include "AutoSize.hpp"
#include <ES2Command/ES2CommandAPI.h>


namespace epsonscan {

void ScanAreaHeight::SetValue(const SDIInt& intVal)
{
	current_ = intVal;
}	
void ScanAreaHeight::Reset()
{
	current_  = 11.4 * 200;
}

void ScanAreaHeight::GetValue(SDIInt& intVal)
{
	intVal = current_;
}

void ScanAreaHeight::GetCapability(SDICapability &capability)
{
	SetDefaultRangeCapability(capability);
}

void ScanAreaHeight::UpdateScanAreaWithDetectedSize(ESFunctionalUnitType funcUnit, ESDictionary& dict)
{
	ST_ES_SIZE_F detectedSize = {0};
	dataProvider_->GetScanner()->GetValueForKey(kESDetectedDocumentSize, detectedSize, funcUnit);

	  //fails to detect
	  if (detectedSize.cx < 1 && detectedSize.cy < 1)
	  {
			dataProvider_->GetScanner()->GetValueForKey(kESMaxScanSize, detectedSize, funcUnit);
	  }
	  dict[kESScanSize] = detectedSize;
}

ESDictionary ScanAreaHeight::GetEngineKeyValues()
{
	auto dict =  ESDictionary();

	SDIInt outputResolution = GetValueInt(kSDIResolutionKey);

	SDIInt scanResolutionX  = dataProvider_->GetCommandValueAsNumber(kESXResolution);
	SDIInt scanResolutionY  = dataProvider_->GetCommandValueAsNumber(kESYResolution);


	// ESC/I機種ではガイドポジションを外部からセットする必要がある
	ESNumber adfOffset = kPrvHRD_ADFOffset_Left;
	if (dataProvider_->GetModelInfo()->GetValue(kPrvHRD_ADFOffset, adfOffset))
	{
		if (adfOffset == kPrvHRD_ADFOffset_Right_Manual)
		{
			dataProvider_->GetScanner()->SetValueForKey(kESGuidePositionADF, (ESNumber)kESGuidePositionRight);
		}
		if (adfOffset == kPrvHRD_ADFOffset_Left)
		{
			dataProvider_->GetScanner()->SetValueForKey(kESGuidePositionADF, (ESNumber)kESGuidePositionLeft);
		}
		if (adfOffset == kPrvHRD_ADFOffset_Center_Auto || adfOffset == kPrvHRD_ADFOffset_Center_Manual)
		{
			dataProvider_->GetScanner()->SetValueForKey(kESGuidePositionADF, (ESNumber)kESGuidePositionCenter);
		}
	}


	SDIInt offsetX = GetValueInt(kSDIScanAreaOffsetXKey) * scanResolutionX / outputResolution;
	SDIInt offsetY = GetValueInt(kSDIScanAreaOffsetYKey) * scanResolutionY / outputResolution;
	SDIInt width = GetValueInt(kSDIScanAreaWidthKey) * scanResolutionX / outputResolution;
	SDIInt height = GetValueInt(kSDIScanAreaHeightKey) * scanResolutionY / outputResolution;

	SDIFunctionalUnit funcUnit = (SDIFunctionalUnit)GetValueInt(kSDIFunctionalUnitKey);
	 auto autoSize = (AutoSize*)(dataProvider_->GetKeyInstance(kSDIAutoSizeKey).get());

	if (funcUnit == kSDIFunctionalUnitFlatbed)
	{		
		if (autoSize->IsDetectedDocumentSizeAvailable(kESFunctionalUnitFlatbed) && GetValueInt(kSDIAutoSizeKey) == kSDIAutoSizeStandard)
		{	
			UpdateScanAreaWithDetectedSize(kESFunctionalUnitFlatbed, dict);
		}
		else{
			dict[kESScanAreaInPixel] = MakeRect<UInt32>(offsetX,
														offsetY,
														offsetX + width,
														offsetY + height);
		}
	}
	else
	{
		if (GetValueInt(kSDIAutoSizeKey) == kSDIAutoSizeStandard)
		{
			 if (autoSize->IsCRPAvailable())
			 {
				dict[kESScanSizeInPixel] = MakeSize<UInt32>(GetValueInt(kSDIMaxScanAreaWidthKey) / 100.0 *  scanResolutionX,
														GetValueInt(kSDIMaxScanAreaHeightKey) / 100.0 * scanResolutionY);
				dict[kESPaperEndDetection] = (bool)false;
			 }
			 else if (autoSize->IsDetectedDocumentSizeAvailable(kESFunctionalUnitDocumentFeeder))
			 {
				 UpdateScanAreaWithDetectedSize(kESFunctionalUnitDocumentFeeder, dict);
				 dict[kESPaperEndDetection] = (bool)true;
			 }else{
				dict[kESScanSizeInPixel] = MakeSize<UInt32>(GetValueInt(kSDIMaxScanAreaWidthKey) / 100.0 *  scanResolutionX,													
															GetValueInt(kSDIMaxScanAreaHeightKey) / 100.0 * scanResolutionY);
														
				dict[kESPaperEndDetection] = (bool)true;
				dict[kESOverScan] = (bool)true;
			 }
			return dict;
		}else if (GetValueInt(kSDIAutoSizeKey) == kSDIAutoSizeLongPaper)
		{
			int  heightMax  =  std::min(120* scanResolutionY, (int)(GetValueInt(kSDIMaxScanAreaHeightInLongPaperKey) / 100.0 * scanResolutionY));
			dict[kESScanSizeInPixel] = MakeSize<UInt32>(GetValueInt(kSDIMaxScanAreaWidthKey) / 100.0 *  scanResolutionX, heightMax);														

			dict[kESPaperEndDetection] = (bool)true;
			dict[kESOverScan] = (bool)true;
			return dict;
		}else if (GetValueInt(kSDIPaperEndDetectionKey) == 1)
		{
			int maxHeight = std::min((int)(GetValueInt(kSDIMaxScanAreaHeightInLongPaperKey)/ 100.0 * scanResolutionY), 215 * 300);
			dict[kESScanSizeInPixel] = MakeSize<UInt32>(width, maxHeight);
														
			dict[kESPaperEndDetection] = (bool)true;
			return dict;
		}

		if (offsetX == 0 && offsetY== 0 )
		{
			dict[kESScanSizeInPixel] = MakeSize<UInt32>(width,
														height);
		}else{
			dict[kESScanAreaInPixel] = MakeRect<UInt32>(offsetX,
														offsetY,
														offsetX + width,
														offsetY + height);
		}
		dict[kESPaperEndDetection] = (bool)false;
	}

	return dict;
}
	
}//namespace epsonscan
